local function GraphDisplay( pn )
	local t = Def.ActorFrame {
		Def.GraphDisplay {
			InitCommand=cmd(Load,"GraphDisplay";);
			BeginCommand=function(self)
				local ss = SCREENMAN:GetTopScreen():GetStageStats();
				self:Set( ss, ss:GetPlayerStageStats(pn) );
			end
		};
		Font("mentone","24px")..{
			Text="NO DATA";
			InitCommand=cmd(diffusealpha,0.8;shadowlength,0);
			BeginCommand=function(self)
				self:visible( not GAMESTATE:IsHumanPlayer(pn) );
			end;
		};
	};
	return t;
end

local function PercentScoreText(pn)
	return Def.ActorFrame{
		Font("mentone","24px")..{
			InitCommand=cmd(shadowlength,1;zoom,0.75;diffuse,PlayerColor(pn));
			BeginCommand=function(self)
				self:horizalign(pn == PLAYER_1 and left or right);
				if GAMESTATE:IsHumanPlayer(pn) then
					-- percent score whores
					local percent = STATSMAN:GetCurStageStats():GetPlayerStageStats(pn):GetPercentDancePoints();
					self:settext(FormatPercentScore(percent));
					local text = self:GetText();
					if text == "100.00%" then self:settext("100%"); end;
				end;
			end;
		};
		-- this label will probably trick people into thinking the graph
		-- represents score instead of life... don't show this. -corporate
		--[[
		Font("mentone","24px")..{
			Text="Score";
			InitCommand=cmd(y,12;shadowlength,1;zoom,0.45;visible,GAMESTATE:IsHumanPlayer(pn));
			BeginCommand=function(self)
				self:horizalign(pn == PLAYER_1 and left or right);
				self:addx(5 * (pn == PLAYER_1 and -1 or 1));
			end;
		};
		--]]
	};
end;

local t = LoadFallbackB();

-- hey why is percent score bundled with graphdisplay still -naket
if ShowStandardDecoration("GraphDisplay") then
	for pn in ivalues(PlayerNumber) do
		t[#t+1] = StandardDecorationFromTable( "GraphDisplay" .. ToEnumShortString(pn), GraphDisplay(pn) );
		t[#t+1] = StandardDecorationFromTable( "PercentScore" .. ToEnumShortString(pn), PercentScoreText(pn) );
	end
end

local function JudgeJerryLynn(pn)
	return LoadActor( THEME:GetPathG("ScreenEvaluation","JudgeLabels"), pn )..{
		Name="JudgeLabels"..ToEnumShortString(pn);
		InitCommand=cmd(hide_if,not GAMESTATE:IsHumanPlayer(pn););
	};
end;

for pn in ivalues(PlayerNumber) do
	--t[#t+1] = StandardDecorationFromFile( "JudgeLabels" .. ToEnumShortString(pn), "JudgeLabels" )..{
	t[#t+1] = StandardDecorationFromTable( "JudgeLabels" .. ToEnumShortString(pn), JudgeJerryLynn(pn) )..{
	};
end;

t[#t+1] = StandardDecorationFromFile( "SongTitleAndArtist", "SongTitleAndArtist" );

if ShowStandardDecoration("StepsDisplay") then
	for pn in ivalues(PlayerNumber) do
		local t2 = Def.StepsDisplay {
			InitCommand=cmd(Load,"StepsDisplayEvaluation",pn;SetFromGameState,pn;);
		};
		t[#t+1] = StandardDecorationFromTable( "StepsDisplay" .. ToEnumShortString(pn), t2 );
	end;
end;

-- record text
local function MachineRecord(pn)
	local stats = STATSMAN:GetCurStageStats():GetPlayerStageStats(pn);
	local bMachineRecord  = (  stats:GetMachineHighScoreIndex() ~= -1 );
	return Font("mentone","24px")..{
		Text=string.format("Machine Record #%i!", stats:GetMachineHighScoreIndex()+1);
		InitCommand=cmd(zoom,0.41667;shadowlength,1;shadowcolor,PlayerDarkColor(pn);diffuse,PlayerColor(pn);NoStroke;glowshift;effectcolor1,color("1,1,1,0");effectcolor2,color("1,1,1,0.25"));
		BeginCommand=cmd(visible,bMachineRecord;);
	};
end;

local function PersonalRecord(pn)
	local stats = STATSMAN:GetCurStageStats():GetPlayerStageStats(pn);
	local bPersonalRecord  = (  stats:GetPersonalHighScoreIndex() ~= -1 );
	return Font("mentone","24px")..{
		Text=string.format("Personal Record #%i!", stats:GetPersonalHighScoreIndex()+1);
		InitCommand=cmd(zoom,0.41667;shadowlength,1;shadowcolor,PlayerDarkColor(pn);diffuse,PlayerColor(pn);NoStroke;glowshift;effectcolor1,color("1,1,1,0");effectcolor2,color("1,1,1,0.25"));
		BeginCommand=cmd(visible,bPersonalRecord;);
	};
end;

if ShowStandardDecoration("RecordText") then
	for pn in ivalues(PlayerNumber) do
		t[#t+1] = StandardDecorationFromTable( "MachineRecord" .. ToEnumShortString(pn), MachineRecord(pn) );
		t[#t+1] = StandardDecorationFromTable( "PersonalRecord" .. ToEnumShortString(pn), PersonalRecord(pn) );
	end
end

-- combo graph
function ComboGraph( pn )
	local t = Def.ActorFrame {
		Def.ComboGraph {
			InitCommand=cmd(Load,"ComboGraph";);
			BeginCommand=function(self)
				local ss = SCREENMAN:GetTopScreen():GetStageStats();
				self:Set( ss, ss:GetPlayerStageStats(pn) );
				self:player( pn );
			end
		};
	};
	return t;
end

if ShowStandardDecoration("ComboGraph") then
	for pn in ivalues(PlayerNumber) do
		t[#t+1] = StandardDecorationFromTable( "ComboGraph"..ToEnumShortString(pn), ComboGraph(pn) );
	end;
end;

return t;