local t = Def.ActorFrame{};

-- Legacy StepMania 4 Function
local function StepsDisplay(pn)
	local function set(self, player)
		self:SetFromGameState( player );
	end

	local sd = Def.StepsDisplay {
		InitCommand=cmd(Load,"StepsDisplay"..PlayerNumberToString(pn),GAMESTATE:GetPlayerState(pn););
		CurrentSongChangedMessageCommand=function(self)
			local hasSelection;
			if GAMESTATE:IsCourseMode() then
				hasSelection = GAMESTATE:GetCurrentCourse() and true or false;
			else
				hasSelection = GAMESTATE:GetCurrentSong() and true or false;
			end;
			self:visible( hasSelection and GAMESTATE:IsHumanPlayer(pn) );
		end;
	};

	if pn == PLAYER_1 then
		sd.CurrentStepsP1ChangedMessageCommand=function(self) set(self, pn); end;
		sd.CurrentTrailP1ChangedMessageCommand=function(self) set(self, pn); end;
	else
		sd.CurrentStepsP2ChangedMessageCommand=function(self) set(self, pn); end;
		sd.CurrentTrailP2ChangedMessageCommand=function(self) set(self, pn); end;
	end

	return sd;
end

for pn in ivalues(PlayerNumber) do
	local MetricsName = "StepsDisplay" .. PlayerNumberToString(pn);
	t[#t+1] = StepsDisplay(pn) .. {
		InitCommand=function(self)
			self:player(pn);
			self:name(MetricsName);
			ActorUtil.LoadAllCommandsAndSetXY(self,Var "LoadingScreen");
			--self:GetChild("Frame"):zoomy(pn == PLAYER_2 and -1 or 1 );
		end;
		PlayerJoinedMessageCommand=function(self, params)
			if params.Player == pn then
				self:visible(true);
				(cmd(zoom,0;bounceend,0.3;zoom,1))(self);
			end;
		end;
		PlayerUnjoinedMessageCommand=function(self, params)
			if params.Player == pn then
				self:visible(true);
				(cmd(bouncebegin,0.3;zoom,0))(self);
			end;
		end;
	};
	t[#t+1] = Font("mentone","24px")..{
		Name="PercentScore"..pn;
		InitCommand=cmd(zoom,0.675;shadowlength,1;player,pn;NoStroke);
		BeginCommand=function(self)
			local xOffset = 128;
			local pX = THEME:GetMetric(Var "LoadingScreen",MetricsName.."X") + (pn == PLAYER_1 and xOffset or -xOffset);
			local pY = THEME:GetMetric(Var "LoadingScreen",MetricsName.."Y") + (pn == PLAYER_1 and 4 or -4);
			self:xy(pX,pY);
			self:horizalign(pn == PLAYER_1 and right or left);
			if pn == PLAYER_1 then
				self:cropright(1)
			else
				self:cropleft(1)
			end;
		end;
		OnCommand=cmd(queuecommand,"Anim";);
		-- handle late join
		PlayerJoinedMessageCommand=function(self, params)
			if params.Player == pn then
				self:linear(0.25)
				if pn == PLAYER_1 then
					self:cropright(0)
				else
					self:cropleft(0)
				end;
				self:playcommand("Set");
			end;
		end;
		PlayerUnjoinedMessageCommand=function(self, params)
			if params.Player == pn then
				self:linear(0.25)
				if pn == PLAYER_1 then
					self:cropright(1)
				else
					self:cropleft(1)
				end;
				self:playcommand("Set");
			end;
		end;
		AnimCommand=function(self)
			self:linear(0.25)
			if pn == PLAYER_1 then
				self:cropright(0)
			else
				self:cropleft(0)
			end;
			self:queuecommand("Set");
		end;
		OffCommand=cmd(linear,0.2;diffusealpha,0);
		SetCommand=function(self)
			local SongOrCourse, StepsOrTrail;

			local scorelist;
			local text = "";
			local profile;

			if GAMESTATE:IsCourseMode() then
				SongOrCourse = GAMESTATE:GetCurrentCourse();
				StepsOrTrail = GAMESTATE:GetCurrentTrail(pn);
			else
				SongOrCourse = GAMESTATE:GetCurrentSong();
				StepsOrTrail = GAMESTATE:GetCurrentSteps(pn);
			end;

			--local dividerScore = -1;
			if SongOrCourse then
				if StepsOrTrail then
					profile = GetProfile(pn);
					scorelist = profile:GetHighScoreList(SongOrCourse,StepsOrTrail);
					assert(scorelist);
					local scores = scorelist:GetHighScores();
					local topscore = scores[1];
					if not topscore then
						--dividerScore = -1
						text = string.format("%.2f%%", 0);
					else
						--dividerScore = topscore:GetPercentDP();
						text = string.format("%.2f%%", topscore:GetPercentDP()*100.0);
					end;
				else
					--dividerScore = -1
					text = string.format("%.2f%%", 0);
				end;
			else
				--dividerScore = -1
				text = "";
			end;
			--MESSAGEMAN:Broadcast("ChangeDividerSize",{PlayerNumber = pn, Score = dividerScore});
			self:settext(text);
		end;
		-- song and course changes
		CurrentSongChangedMessageCommand=cmd(playcommand,"Set");
		CurrentCourseChangedMessageCommand=cmd(playcommand,"Set");
		-- steps and trail changes
		CurrentStepsP1ChangedMessageCommand=function(self)
			if pn == PLAYER_1 then self:playcommand("Set"); end;
		end;
		CurrentStepsP2ChangedMessageCommand=function(self)
			if pn == PLAYER_2 then self:playcommand("Set"); end;
		end;

		CurrentTrailP1ChangedMessageCommand=function(self)
			if pn == PLAYER_1 then self:playcommand("Set"); end;
		end;
		CurrentTrailP2ChangedMessageCommand=function(self)
			if pn == PLAYER_2 then self:playcommand("Set"); end;
		end;
		-- hacks for handling player divider/meter
		NextSongMessage=cmd(queuecommand,"Set");
		PreviousSongMessage=cmd(queuecommand,"Set");
	};
end;

if GAMESTATE:IsCourseMode() then
	t[#t+1] = LoadActor("Bottom_Courses")..{
		InitCommand=cmd(hide_if,not GAMESTATE:IsCourseMode());
	};
else
	t[#t+1] = LoadActor("Bottom_Songs")..{
		InitCommand=cmd(hide_if,GAMESTATE:IsCourseMode());
	};
end;

--[[  BannerArea ]]
t[#t+1] = LoadActor("BannerArea")..{
	-- 204,93
	-- SCREEN_WIDTH*0.264583 is perfect on 960x600, but for 4:3 we want else.
	-- was screen center x * 0.53
	InitCommand=cmd(x,WideScale(SCREEN_WIDTH*0.225,SCREEN_WIDTH*0.264583);y,SCREEN_CENTER_Y*0.3875;fov,90);
};
--[[ /BannerArea ]]

t[#t+1] = LoadActor("ArrowOverlay")..{
	InitCommand=cmd(x,THEME:GetMetric("ScreenSelectMusic","MusicWheelX");y,SCREEN_CENTER_Y);
};

--[[ overlay zone below this line ]]
t[#t+1] = Font("mentone","24px")..{
	Text="Press &START; for options"; -- todo: stringify
	InitCommand=cmd(x,SCREEN_CENTER_X;y,SCREEN_CENTER_Y*1.25;vertalign,bottom;NoStroke;shadowlength,1;shadowcolor,color("0,0,0,0.375"));
	OnCommand=cmd(visible,false);
	ShowPressStartForOptionsCommand=cmd(visible,true;zoom,1.5;decelerate,1;zoom,1);
	ShowEnteringOptionsCommand=cmd(settext,"Entering Options...";hurrytweening,0.1;sleep,1;accelerate,0.1;zoomy,0);
	OffCommand=cmd(sleep,0.75;accelerate,0.25;zoomx,0);
};
return t;