--[[
NAKET Coder presents a combo display
featuring a few things not seen much.
]]
local c;
local player = Var "Player";
local ShowComboAt = THEME:GetMetric("Combo", "ShowComboAt");

local curCombo, lastCombo;
local toastyBarWidth = 128;
local toastyBarHeight = 10; -- was 16
local gotToasty = false;

local numlblOffset = 24;
--[[
the naket coder list of values that work great (outdated):

minZoom = 0.5
maxZoom = 0.9 (max at 500),
          1.0 (max at 999; provides the best compromise),
          1.75 (max at 999, but you'll have to go down for numbers > 999)
--]]
local NumberMinZoom = 0.5; -- y should be 62 (36+26). 62/36 = 1.72222 (@0.5)
local NumberMaxZoom = 0.9; -- y should be numlblOffset for 0.9...
local NumberMaxZoomAt = 999;

-- if a song/trail has less than 999 max combo, then change the combo meter
-- to grow to the max combo.
local curStepsOrTrail;
if GAMESTATE:IsCourseMode() then
	curStepsOrTrail = GAMESTATE:GetCurrentTrail(player);
else
	curStepsOrTrail = GAMESTATE:GetCurrentSteps(player);
end;

if curStepsOrTrail then
	local rv = curStepsOrTrail:GetRadarValues(player);
	local taps = rv:GetValue('RadarCategory_TapsAndHolds');
	local jumps = rv:GetValue('RadarCategory_Jumps');
	if not THEME:GetMetric("Gameplay","ComboIsPerRow") then
		taps = taps + jumps;
	end;
	if taps < NumberMaxZoomAt then
		NumberMaxZoomAt = taps;
	end;
end;

local flags = { Judge = false; Combo = false; };

local t = Def.ActorFrame {
	LoadFont( "Combo", "numbers" ) .. {
		Name="Number";
		OnCommand=cmd(y,numlblOffset;shadowlength,1;vertalign,bottom);
	};
	LoadFont( "Combo", "numbers" ) .. {
		Name="Number2";
		OnCommand=cmd(y,numlblOffset;shadowlength,0;vertalign,bottom);
	};
	LoadFont( "Combo", "glow" ) .. {
		Name="Number3";
		OnCommand=cmd(y,numlblOffset;shadowlength,0;vertalign,bottom);
	};
	LoadActor("_flare blue")..{
		Name="Flasher";
		InitCommand=cmd(diffusealpha,0);
		OnCommand=cmd(blend,'BlendMode_Add',diffusealpha,0);
		ShowCommand=cmd(zoom,1;accelerate,0.5;diffusealpha,1;zoomx,1.5;zoomy,1.5;decelerate,0.5;zoomx,4;zoomy,3;diffusealpha,0);
	};

	Def.ActorFrame{
		Name="ComboLabelFrame";
		OnCommand=cmd(y,numlblOffset*1.77778);

		LoadActor("_label base") .. {
			Name="ComboLabelBase";
		};

		LoadActor("_label bevel") .. {
			Name="ComboLabelBevel";
		};
	};

	Def.Quad{
		Name="ToastyMeterBG";
		InitCommand=cmd(zoomto,toastyBarWidth+2,toastyBarHeight+2;y,numlblOffset*3.16667;vertalign,bottom;diffuse,color("#000000");diffusebottomedge,color("#444444"));
	};

	Def.Quad{
		Name="ToastyMeterFill";
		InitCommand=cmd(zoomto,0,toastyBarHeight;x,-(toastyBarWidth/2);y,numlblOffset*3.125;vertalign,bottom;horizalign,left;diffusebottomedge,PlayerColor(player));
	};

	Font("mentone","24px")..{
		Name="ToastyComboNumber";
		Text="0/250";
		InitCommand=cmd(x,toastyBarWidth*0.485;y,numlblOffset*2.9;zoom,0.5;shadowlength,1;horizalign,right;diffuse,color("1,1,1,0.75"));
	};

	InitCommand = function(self)
		c = self:GetChildren();
		c.Number:visible(false);
		c.Number2:visible(false);
		c.Number3:visible(false);
		c.ComboLabelFrame:visible(false);
		c.ToastyMeterBG:visible(false);
		c.ToastyMeterFill:visible(false);
		c.ToastyComboNumber:visible(false);
	end;

--[[
	Order of commands during runtime:
	02:48.975: Judgment runs
	02:48.975: ComboChanged runs
	02:48.975: Combo runs
	They all happen in the same millisecond, but by placing the print statements,
	they appear in that order consistently, so I have no choice but to accept
	the above order as how StepMania runs it. -tcoder

	What this means is rethinking the strategy.
	ComboChanged runs more than either of the two commands. In fact,
	ComboChanged fires off multiple times, when not needed....

	ComboChanged:
	* Take care of the old combos...

	Judgment:
	* Figure out if a step is worthy of being considered toasty bait
	* diffuse the glow by the judgment color
	+ judge flag

	ComboChanged pass 2:
	* Take care of the old combos...
	* ...and if there's toasty bait, you have to wait.

	Combo:
	* combo man, combo man, does whatever the combo man does
	* including taking care of the current combo.
	+ combo flag

	ComboChanged pass 3:
	* Take care of the old combos...
	* ...and the normal combo.
	* If there's toasty bait, check it. by now, we have everything we need.

	flags["Judge"] and flags["Combo"]

	on second thought, this is primarily for sm-ssc so fuck it:
	ScoreChangedMessageCommand. One of the new params is ToastyCombo.
	YOU DO THE MATH!!!!! fuck'n bullshit
--]]
	ComboChangedMessageCommand=function(self, param)
		-- this function runs the most often.
		if param.Player ~= player then return end;
		lastCombo = param.OldCombo;
	end;

	ComboCommand=function(self, param)
		local iCombo = param.Misses or param.Combo;
		if not iCombo or iCombo < ShowComboAt then
			c.Number:visible(false);
			c.Number2:visible(false);
			c.Number3:visible(false);
			c.ComboLabelFrame:visible(false);
			c.ToastyMeterBG:visible(false);
			c.ToastyMeterFill:visible(false);
			c.ToastyComboNumber:visible(false);
			return;
		end

		local Label = c.ComboLabelFrame;
		local LabelBase = c.ComboLabelFrame:GetChild("ComboLabelBase");
		local bComboOrMiss = false;
		if param.Combo then
			LabelBase:diffusecolor( color("#fff5cc") );

			if SSC then
				c.ToastyMeterBG:visible(true);
				c.ToastyMeterFill:visible(true);
				c.ToastyComboNumber:visible(true);
			end;

			bComboOrMiss = true;
		elseif param.Misses then
			LabelBase:diffuse( color("#ffcccc") );
			bComboOrMiss = true;
		end

		param.Zoom = scale( iCombo, 0, NumberMaxZoomAt, NumberMinZoom, NumberMaxZoom );
		param.Zoom = clamp( param.Zoom, NumberMinZoom, NumberMaxZoom );

		if bComboOrMiss then
			c.Number:visible(true);
			c.Number2:visible(true);
			c.Number3:visible(true);
			Label:visible(true);
		end;

		local comboText = string.format("%03i", iCombo);
		curCombo = iCombo; -- update combo

		c.Number:zoom( param.Zoom );
		c.Number:settext( comboText );

		-- phase two: the add
		c.Number2:zoom( param.Zoom );
		c.Number2:settext( comboText );
		c.Number2:diffusecolor( color("1,0.8,0,1") );
		c.Number2:blend('BlendMode_Add');
		c.Number2:diffusealpha(0.15);

		-- and number 3 (glow)
		c.Number3:zoom( param.Zoom );
		c.Number3:settext( comboText );
		c.Number3:diffusealpha(0.75);

		-- this function glows the combo number set.
		local Pulse = function(self)
			self:stoptweening();

			c.Number2:stoptweening();
			c.Number3:stoptweening();

			local GlowStart;
			local GlowEnd;
			local StartAlpha = 0.425;
			if param.Combo then
				GlowColor = color("1,1,1,"..StartAlpha);
				GlowEnd = color("1,1,1,0");
			else
				GlowColor = color("1,0,0,"..StartAlpha);
				GlowEnd = color("1,0,0,0");
			end;

			c.Number2:glow(GlowColor);
			c.Number2:smooth(0.15);
			c.Number2:glow(GlowEnd);

			c.Number3:decelerate(0.15);
			c.Number3:diffusealpha(0);
		end;

		Pulse( c.Number, param );

		-- every 100, fire off this flash somehow.
		-- xxx: this doesn't work if you have, say 299 and a jump comes up.
		if iCombo > 75 and iCombo % 100 == 0 then
			c.Flasher:visible(true);
			c.Flasher:playcommand("Show");
		end;

		-- don't show toasty meter if easter eggs are disabled
		-- also don't show it in non-sm-ssc since the method we use only works
		-- in sm-ssc.
		if PREFSMAN:GetPreference("EasterEggs") == 0 or not SSC then
			c.ToastyMeterBG:visible(false);
			c.ToastyMeterFill:visible(false);
			c.ToastyComboNumber:visible(false);
		end;
		flags["Combo"] = true;
	end;

	ScoreChangedMessageCommand=function(self,param)
		-- non-toasty related things. (always play these)
		if tns then
			c.Number3:diffusecolor( JudgeColor(tns) );
		end;

		-- toasty-related things
		local toastyCombo = param.ToastyCombo;
		if toastyCombo and toastyCombo >= 250 then
			toastyCombo = -1;
			gotToasty = true;
		end;

		-- i don't care that triggers exist, they don't work correctly. -nc
		if gotToasty then
			-- bye
			c.ToastyMeterFill:stoptweening();
			c.ToastyMeterFill:bouncebegin(1);
			c.ToastyMeterFill:zoomy(0);
			c.ToastyMeterBG:bouncebegin(1);
			c.ToastyMeterBG:zoomy(0);
			c.ToastyComboNumber:bouncebegin(1);
			c.ToastyComboNumber:zoomy(0);
			return;
		end;

		-- special case: if going to 0, animate down.
		if toastyCombo and SSC then
			if c.ToastyMeterFill:GetZoomX() > 0 and toastyCombo == 0 then
				c.ToastyMeterFill:stoptweening();
				c.ToastyMeterFill:decelerate(1);
				c.ToastyMeterFill:zoomx(0);
			else
				c.ToastyMeterFill:zoomx( (toastyCombo/250) * toastyBarWidth );
			end;
			c.ToastyComboNumber:settext(toastyCombo.."/250");
		end;
	end;

	JudgmentMessageCommand=function(self, param)
		if param.Player ~= player then return end;
		if not param.TapNoteScore then return end;
		if param.HoldNoteScore then return end; -- eat shit stepmania!!!

		local tns = param.TapNoteScore;
	end;

	OffCommand=function(self)
		-- so yeah this is where the pets go
		-- you should probably check for a full combo here.
		local curPlayerStats = STATSMAN:GetCurStageStats():GetPlayerStageStats(player);
		local bFullCombo = curPlayerStats:CompatibleFullCombo();

		if bFullCombo then
			-- begin state
			c.Number2:diffusecolor( color("1,0.8,0,1") );
			c.Number2:blend('BlendMode_Add');
			c.Number2:glow(PlayerColor(player));
			c.Number2:diffusealpha(0.15);

			c.Number3:diffusealpha(0.75);

			-- animate
			c.Number2:accelerate(0.5);
			c.Number2:diffusealpha(0.85);
			c.Number2:zoom(1.25);
			c.Number2:sleep(0.1);

			-- end state
			c.Number2:decelerate(0.75);
			c.Number2:glow(Alpha(PlayerColor(player),0));
			c.Number2:zoom(1.5);
			c.Number2:diffusealpha(0);

			c.Number3:decelerate(0.75);
			c.Number3:diffusealpha(0);
		end;
	end;
};

return t;