local index = Var("GameCommand"):GetIndex();
local text = Var("GameCommand"):GetText();

local LineSpacing = 96;
local VertHeight=SCREEN_CENTER_Y;
local iconChangeFocusSeconds = 0.15; -- was 0.05

local OptionColors = {
	HSVA(128, 0.6, 0.275, 0.53125),	-- play
	HSVA(192, 0.6, 0.275, 0.53125),	-- options
	HSVA( 64, 0.6, 0.275, 0.53125),	-- edit
	HSVA(  0, 0.6, 0.275, 0.53125),	-- exit
	-------------------------------------
	color("#44220088"), -- Start Game [O]
	color("#00440088"), -- Options    [G]
	color("#00004488"), -- Edit       [B]
	color("#44000088"), -- Exit       [R]
	-------------------------------------
	color("#44440088"), -- Gametypes  [Y]
	color("#44004488"), -- Jukebox    [V]
};

local OptionColorsFocused = {
	HSVA(128, 1, 0.45, 0.8),	-- play
	HSVA(192, 1, 0.45, 0.8),	-- options
	HSVA( 64, 1, 0.45, 0.8),	-- edit
	HSVA(  0, 1, 0.45, 0.8),	-- exit
	-------------------------------------
	color("#663300CC"), -- Start Game [O]
	color("#006600CC"), -- Options    [G]
	color("#000066CC"), -- Edit       [B]
	color("#660000CC"), -- Exit       [R]
	-------------------------------------
	color("#666600CC"), -- Gametypes  [Y]
	color("#660066CC"), -- Jukebox    [V]
};

local OptionColorsBright = {
	HSVA(128,0.6,0.85,1),	-- play
	HSVA(192,0.6,0.85,1),	-- options
	HSVA(64,0.6,0.85,1),	-- edit
	HSVA(0,0.6,0.85,1),	-- exit
	-------------------------------------
	color("#AA5500FF"), -- Start Game [O]
	color("#00AA00FF"), -- Options    [G]
	color("#0000AAFF"), -- Edit       [B]
	color("#AA0000FF"), -- Exit       [R]
	-------------------------------------
	color("#AAAA00FF"), -- Gametypes  [Y]
	color("#AA00AAFF"), -- Jukebox    [V]
};

local rot = { x = 0, y = 0, z = 0 };

local t = Def.ActorFrame{
	-- vert lines
	Def.Quad{
		InitCommand=cmd(x,-LineSpacing;zoomto,2,VertHeight;fadetop,0.25;fadebottom,0.25);
		GainFocusCommand=cmd(finishtweening;linear,0.2;diffuse,color("#FFF186"));
		LoseFocusCommand=cmd(finishtweening;linear,0.2;diffuse,color("#FFFFFF"));
		OffFocusedCommand=cmd(accelerate,0.15;zoomx,4;sleep,0.5;decelerate,0.5;diffusealpha,0);
		OffUnfocusedCommand=cmd(accelerate,0.15;zoomy,0);
	};
	Def.Quad{
		InitCommand=cmd(x,LineSpacing;zoomto,2,VertHeight;fadetop,0.25;fadebottom,0.25);
		GainFocusCommand=cmd(finishtweening;linear,0.2;diffuse,color("#FFF186"));
		LoseFocusCommand=cmd(finishtweening;linear,0.2;diffuse,color("#FFFFFF"));
		OffFocusedCommand=cmd(accelerate,0.15;zoomx,4;sleep,0.5;decelerate,0.5;diffusealpha,0);
		OffUnfocusedCommand=cmd(accelerate,0.15;zoomy,0);
	};

	-- before BG
	Def.ActorFrame{
		Name="Tex Murphy"; -- http://en.wikipedia.org/wiki/Tex_Murphy
		InitCommand=cmd(fov,45);
		GainFocusCommand=cmd(linear,0.2;diffusealpha,0.5);
		LoseFocusCommand=cmd(stoptweening;diffusealpha,0);
		OffCommand=cmd(runcommandsonleaves,cmd(decelerate,2;diffusealpha,0));

		-- texcoordvelocity: -1 to the right, 1 to the left.

		-- center wall 1: top
		LoadActor( THEME:GetPathG("","_tex") )..{
			InitCommand=cmd(y,-4;z,-94;zoomto,162,VertHeight*0.85;texcoordvelocity,0,0.125;cropbottom,0.5);
		};
		-- center wall 2: bottom
		LoadActor( THEME:GetPathG("","_tex") )..{
			InitCommand=cmd(y,-4;z,-94;zoomto,162,VertHeight*0.85;texcoordvelocity,0,-0.125;croptop,0.5);
		};

		-- floor
		LoadActor( THEME:GetPathG("","_tex") )..{
			InitCommand=cmd(z,0;y,100;zoomto,163,VertHeight;texcoordvelocity,0,0.25;rotationx,-90);
		};
		-- floor shadow
		--[[
		Def.Quad{
			InitCommand=cmd(z,0;y,100;zoomto,163,VertHeight;rotationx,-90;diffusecolor,color("0,0,0");diffusealpha,0.75;fadebottom,0.5);
		};
		--]]

		-- left side
		LoadActor( THEME:GetPathG("","_tex") )..{
			InitCommand=cmd(x,-86;y,-1;z,-33;zoomto,244,VertHeight*0.9125;texcoordvelocity,-0.25,0;rotationy,-90);
		};

		-- right side
		LoadActor( THEME:GetPathG("","_tex") )..{
			InitCommand=cmd(x,86;y,-1;z,-33;zoomto,244,VertHeight*0.9125;texcoordvelocity,0.25,0;rotationy,90);
		};

		-- ceiling
		LoadActor( THEME:GetPathG("","_tex") )..{
			InitCommand=cmd(z,0;y,-106;zoomto,164,VertHeight;texcoordvelocity,0,-0.25;rotationx,90);
		};
		--[[
		-- ceiling shadow
		Def.Quad{
			InitCommand=cmd(z,0;y,-106;zoomto,164,VertHeight;rotationx,90;diffusecolor,color("0,0,0");diffusealpha,0.75;fadetop,0.5);
		};
		--]]
	};

	-- item background
	Def.Quad{
		InitCommand=cmd(zoomto,190,VertHeight;fadetop,0.25;diffusecolor,OptionColors[index+1];fadebottom,0.25;blend,'BlendMode_WeightedMultiply');
		GainFocusCommand=cmd(stoptweening;decelerate,0.75;diffusecolor,OptionColorsFocused[index+1]);
		LoseFocusCommand=cmd(stoptweening;accelerate,0.3;diffusecolor,OptionColors[index+1]);

		OffFocusedCommand=function(self)
			MESSAGEMAN:Broadcast("TitleMenuOff",{ SelectedIndex = index, Color = OptionColorsFocused[index+1] });
			self:sleep(0.5);
			self:decelerate(0.5);
			self:zoomy(0);
		end;
		OffUnfocusedCommand=cmd(accelerate,0.5;zoomy,0);
	};
	-- item bg shine
	Def.Quad{
		--InitCommand=cmd(zoomto,190,20;y,-(VertHeight/2);vertalign,top;diffuse,color("1,1,1,0");fadebottom,0.25;blend,'BlendMode_Add');
		InitCommand=cmd(zoomto,190,20; y,-(VertHeight/2);vertalign,top;
						diffuse,OptionColorsBright[index+1];
						fadetop,0.15;
						fadebottom,0.75;
						--diffusebottomedge,color("1,1,1,0.5");
						blend,'BlendMode_Add'
		);
		GainFocusCommand=cmd(diffusealpha,0.5;linear,0.2;addy,VertHeight/2;diffusealpha,0);
		LoseFocusCommand=cmd(stoptweening;diffusealpha,0;y,-(VertHeight/2));
	};

	-- icon base
	LoadActor( THEME:GetPathG("","_icon") )..{
		InitCommand=cmd(diffuse,OptionColorsBright[index+1]);
		GainFocusCommand=cmd(spring,iconChangeFocusSeconds;addy,-8);
		LoseFocusCommand=cmd(stoptweening;spring,iconChangeFocusSeconds;y,0);

		OffFocusedCommand=cmd(decelerate,0.5;zoom,1.25;sleep,0.5;decelerate,0.5;zoom,0;diffusealpha,0);
		OffUnfocusedCommand=cmd(spin;accelerate,0.5;zoom,0);
	};

	-- image
	LoadActor( THEME:GetPathG("_title","images/"..text) )..{
		GainFocusCommand=cmd(spring,iconChangeFocusSeconds;diffusealpha,1;addy,-8);
		LoseFocusCommand=cmd(stoptweening;spring,iconChangeFocusSeconds;diffusealpha,0.05;y,0);
		OffCommand=cmd(decelerate,0.5;zoom,1.25;sleep,0.5;decelerate,0.5;zoom,0;diffusealpha,0);
	};

	-- title
	--Font("sf theramin gothic", "36px") ..{
	Font("mentone", "24px") ..{
		Name="Choice"..index;
		Text=THEME:GetString( 'ScreenTitleMenu', text );
		InitCommand=cmd(shadowlength,2;strokecolor,color("#22222244"));
		GainFocusCommand=cmd(linear,iconChangeFocusSeconds;diffuse,color("#FFFFFF");y,8;glowshift;effectcolor2,1,0.9,0.33,0.25);
		LoseFocusCommand=cmd(stoptweening;linear,iconChangeFocusSeconds;diffuse,color("#888888");y,0;stopeffect);
		DisabledCommand=cmd(diffuse,0.5,0.5,0.5,1);
		OffFocusedCommand=cmd(decelerate,0.5;zoom,1.25;sleep,0.5;decelerate,0.5;diffusealpha,0);
		OffUnfocusedCommand=cmd(accelerate,0.5;diffusealpha,0);
	};

	-- explanation
	--Font("handelgothic", "20px") ..{ -- zoom,0.75
	--Font("helvetica", "uc 24px") ..{ -- zoom,1.5
	--Font("mentone", "24px") ..{ -- zoom,1.0
	Font("milibus","sb 20px")..{
		Name="Explain"..index;
		-- was y,64, then y,SCREEN_CENTER_Y*0.7
		InitCommand=cmd(shadowlength,1;y,SCREEN_CENTER_Y*0.65;zoom,1.125;wrapwidthpixels,256;diffusebottomedge,color("#CCCCCC");strokecolor,color("#88888844"));
		BeginCommand=function(self)
			local explText = THEME:GetString( 'ScreenTitleMenu', text.."-Explanation" );
			if text == "Exit" then
				explText = string.format(THEME:GetString( 'ScreenTitleMenu', text.."-Explanation" ), ProductID());
			end;
			self:settext( explText );
		end;
		GainFocusCommand=cmd(finishtweening;linear,0.5;diffusealpha,1);
		LoseFocusCommand=cmd(finishtweening;linear,0.5;diffusealpha,0);
		OffCommand=cmd(accelerate,0.5;diffusealpha,0);
	};
};

t[#t+1] = Def.Actor{
	Name="TitleController";
	GainFocusCommand=function(self)
		MESSAGEMAN:Broadcast("TitleGainFocus", { GainedFocus = index });
	end;
	LoseFocusCommand=function(self)
		MESSAGEMAN:Broadcast("TitleLoseFocus", { LostFocus = index });
	end;
	OffFocusedCommand=function(self)
		MESSAGEMAN:Broadcast("OffFocused", { Focused = index });
	end;
	OffUnfocusedCommand=function(self)
		MESSAGEMAN:Broadcast("OffUnfocused", { Unfocused = index });
	end;
};

return t;