-- naket library is where we put all the functions that only make sense in ncrx
-- or have other useful snippets that we don't want to put elsewhere.

function Actor:SecondaryMask(self)
	self:clearzbuffer(false);
	self:zwrite(true);
	self:blend('BlendMode_NoEffect');
end;

-- according to Google, 1 degree = 0.0174532925 radians
function DegreesToRadians(degrees)
	return degrees * 0.0174532925;
end;

function GetProfile(pn)
	if PROFILEMAN:IsPersistentProfile(pn) then
		-- player profile
		return PROFILEMAN:GetProfile(pn);
	else
		-- machine profile
		return PROFILEMAN:GetMachineProfile();
	end;
end;

-- Center1PDependent(normalVal,center1pVal)
-- Checks if the conditions are right for Center1Player to be active and returns
-- a value based on that.
function Center1PDependent(normalVal,center1pVal)
	if PREFSMAN:GetPreference("Center1Player") and GAMESTATE:GetNumPlayersEnabled() == 1 then
		return center1pVal;
	else
		return normalVal;
	end;
end

-- deals with options cursor and underline crap;
function Actor:OptionsCursor(pn)
	if SSC then
		self:diffuse(PlayerColor(pn));
	end;
end;
-- sm4 doesn't support custom diffuse on underline/cursor yet.
function OptionsCursor(pn)
	if not SSC then
		self:diffuse(color("1,1,1,1"))
	end;
end;

-- used for sort orders
function Sorts()
	local sorts = {
		"SortOrder_Preferred",
		"SortOrder_Group",
		"SortOrder_Title",
		"SortOrder_BPM",
		"SortOrder_Popularity",
		"SortOrder_Artist",
		"SortOrder_Genre"
	};
	if SSC then
		sorts[#sorts+1] = "SortOrder_Recent";
	end;
	return sorts;
end;

function SortNames()
	local strSort = "Preferred,Group,Title,Bpm,Popularity,TopGrades,Artist,EasyMeter,MediumMeter,HardMeter,Genre,Length";
	if SSC then
		strSort = strSort .. ",Recent";
	end;
	return strSort;
end;

function PlayerOptionsMainChoices()
	-- in certain situations, some options should be disabled.
	local pm = GAMESTATE:GetPlayMode()
	if pm == 'PlayMode_Oni' then
		return "SysDir,PlaySong";
	end;
	return "Mods,SongOpts,SysDir,PlaySong"; -- default
end;

-- thanks AJ (totally ganked this from moonlight; switched up comments some)
function GetStepManiaAlphaVersion()
	local version = ProductVersion();
	-- if it matches "alpha #", use #.
	local startIdx,endIdx = string.find(version, "alpha ");
	if endIdx then
		version = string.sub(version,endIdx+1);
	end;
	return tonumber(version);
end;

function PlayerStageStats:CompatibleFullCombo()
	local tnsComboThreshold = THEME:GetMetric("Gameplay","MinScoreToContinueCombo")
	if SSC then
		-- sm-ssc has FullComboOfScore(tns)
		return self:FullComboOfScore(tnsComboThreshold)
	else
		-- SM4 alpha 6 has various IsFullComboW* bindings.
		-- doing this when you could use FullComboOfScore is beyond me. -naket
		if GetStepManiaAlphaVersion() >= 6 then
			local FullComboFuncs = {
				TapNoteScore_W1 = self:IsFullComboW1(),
				TapNoteScore_W2 = self:IsFullComboW2(),
				TapNoteScore_W3 = self:IsFullComboW3(),
				TapNoteScore_W4 = self:IsFullComboW4(),
				-- only goes up to W4
			};
			return FullComboFuncs[tnsComboThreshold];
		else
			-- SM4 alpha 5 and below supports FullCombo(), which is bound to
			-- TapNoteScore_W3 and above.
			return self:FullCombo();
		end;
	end;
	return nil -- we shouldn't get down here.
end;

-- only used for "out of order" bit:
function RandomSound()
	local sounds = {
		"_YOU", "_dispenser"
	};
	return sounds[math.random(#sounds)]
end;