--[[ StepMania Online Branch Helpers  ======================================== ]]

-- NextNetworkScreen()
-- returns the next screen to go to in ScreenNetworkOptions.
function NextNetworkScreen()
	if IsNetConnected() then
		-- successfully connected.
		return SMOnlineScreen()
	end
	-- otherwise it goes back to ScreenTitleMenu.
	return "ScreenTitleMenu"
end

-- GetModeOnlineNext()
-- Returns the next screen when selecting Online on ScreenSelectGameMode.
function GetModeOnlineNext()
	if IsNetConnected() then
		-- check login to SMO
		return SMOnlineScreen()
	end
	-- connect to a server
	return "ScreenNetworkOptions"
end

function SMOnlineScreen()
	local function NotLoggedIn(pn)
		return not IsSMOnlineLoggedIn(pn) and GAMESTATE:IsPlayerEnabled(pn);
	end;
	if NotLoggedIn(PLAYER_1) or NotLoggedIn(PLAYER_2) then
		return "ScreenSMOnlineLogin";
	end;
	return "ScreenNetRoom";
end

--[[ Branches =============================================================== ]]
if Branch then
	-- using sm-ssc:
	Branch.PostExtra = function()
		-- currently there's no way to pick doubles for courses.
	end;
	Branch.PlayerOptions = function()
		local optionsScreen = "ScreenPlayerOptionsMain"
		local nextScreen = "ScreenStageInformation";
		return (SCREENMAN:GetTopScreen():GetGoToOptions()) and optionsScreen or nextScreen;
	end;
	Branch.SelMusic = function()
		local pm = GAMESTATE:GetPlayMode()

		if pm ~= "PlayMode_Regular" and IsNetConnected() then
			SCREENMAN:SystemMessage( THEME:GetString("NetworkSyncManager","Not Supported") )
			return "ScreenSelectGameMode"
		end

		if pm == "PlayMode_Nonstop" or pm == "PlayMode_Oni" or pm == "PlayMode_Endless"	then
			return "ScreenSelectCourse"
		end

		if IsNetConnected() then ReportStyle() end
		if IsNetSMOnline() then return SMOnlineScreen() end
		if IsNetConnected() then return "ScreenNetSelectMusic" end
		return "ScreenSelectMusic"
	end;
	Branch.Gameplay = function()
		local st = GAMESTATE:GetCurrentStyle():GetStyleType()
		if st == "StyleType_TwoPlayersSharedSides" then return "ScreenGameplayShared" end
		return "ScreenGameplay"
	end;
	Branch.NetworkOptions2 = function()
		if IsNetSMOnline() then return "ScreenReportStyle" end
		return "ScreenNetworkOptions2";
	end;
	Branch.NetworkOptions2Next = function()
		if IsNetConnected() then return "ScreenReportStyle" end
		if IsNetSMOnline() then return SMOnlineScreen() end
		return "ScreenSelectGameMode"; -- exit option and not online
	end;
	Branch.ReportStyle = function()
		if IsNetConnected() then ReportStyle() end
		if IsNetSMOnline() then return SMOnlineScreen() end
	end;
else
	-- not running sm-ssc, have to define branch table since I'm not gonna redo
	-- the metrics for the SM4 version. -naket
	Branch = {
		-- sm-ssc
		TitleMenu = "ScreenTitleMenu",
		AfterProfileSave = function()
			if GAMESTATE:IsEventMode() then
				return Branch.SelMusic();
			elseif STATSMAN:GetCurStageStats():AllFailed() then
				return "ScreenTitleIntro"
			elseif GAMESTATE:GetSmallestNumStagesLeftForAnyHumanPlayer() == 0 then
				return "ScreenTitleIntro"
			else
				return Branch.SelMusic();
			end
		end,

		-- ncrx
		SelMusic = function()
			local pm = GAMESTATE:GetPlayMode()

			if pm ~= "PlayMode_Regular" and IsNetConnected() then
				SCREENMAN:SystemMessage( THEME:GetString("NetworkSyncManager","Not Supported") )
				return "ScreenSelectGameMode"
			end

			if pm == "PlayMode_Nonstop" or pm == "PlayMode_Oni" or pm == "PlayMode_Endless"	then
				return "ScreenSelectCourse"
			end

			if IsNetConnected() then ReportStyle() end
			if IsNetSMOnline() then return SMOnlineScreen() end
			if IsNetConnected() then return "ScreenNetSelectMusic" end
			return "ScreenSelectMusic"
		end,
		PlayerOptions = function()
			local optionsScreen = "ScreenPlayerOptionsMain"
			local nextScreen = "ScreenStageInformation";
			return (SCREENMAN:GetTopScreen():GetGoToOptions()) and optionsScreen or nextScreen;
		end;
		Gameplay = function()
			local st = GAMESTATE:GetCurrentStyle():GetStyleType()
			if st == "StyleType_TwoPlayersSharedSides" then return "ScreenGameplayShared" end
			return "ScreenGameplay"
		end;
		NetworkOptions2 = function()
			if IsNetSMOnline() then return "ScreenReportStyle" end
			return "ScreenNetworkOptions2";
		end;
		NetworkOptions2Next = function()
			if IsNetConnected() then return "ScreenReportStyle" end
			if IsNetSMOnline() then return SMOnlineScreen() end
			return "ScreenSelectGameMode"; -- exit option and not online
		end;
		ReportStyle = function()
			if IsNetConnected() then ReportStyle() end
			if IsNetSMOnline() then return SMOnlineScreen() end
		end;
	};
end;

-- the following functions need to be converted for sm-ssc and stuff
function GetServiceOptionsScreen()
	-- can you ever make up your mind y/n
	return "ScreenOptionsService";
	--return "ScreenOptionsService2";
end;